/*
* Copyright 2004-2005 mbsx@seznam.cz
* All Rights Reserved.
*/
using System;
using System.Drawing;
using System.Drawing.Printing;
using System.IO;
using System.Windows.Forms;

namespace DeskNotes
{
	/// <summary>
	/// Summary description for NotePrintDocument.
	/// </summary>
	public class NotePrintDocument : PrintDocument
	{
		const int BOTTOM_PAGE = 5;
					
		private IPrintableNote[] notes;
		private Font defaultPrintFont = new Font("Arial", 10);
		private int noteID;
		private int pageID;
		private string textForNextPage = "";
		static private int ticks = System.Environment.TickCount;
		private StringFormat stringFormat = new StringFormat(StringFormat.GenericTypographic);
		
		internal Font DefaultFont
		{
			set 
			{
				defaultPrintFont = value;
			}
		}

		internal NotePrintDocument(PageSettings pageSettings, IPrintableNote[] notes)
		{
			
			//PrintTicks("NotePrintDoc <init>: ");
			this.DefaultPageSettings = pageSettings;
			this.notes = notes;
			this.DocumentName = MainResource.GetString("APP_TITLE");
			//PrintTicks("NotePrintDoc <inited>: ");
		}

		static private void PrintTicks(string text)
		{
			int tick = ticks;
			ticks = System.Environment.TickCount;
			System.Console.WriteLine(text + (ticks - tick) );
		}

		internal NotePrintDocument(PageSettings pageSettings, IPrintableNote note)
		{
			
			this.DefaultPageSettings = pageSettings;
			this.notes = new IPrintableNote[1];
			this.notes[0] = note;
			this.DocumentName = MainResource.GetString("APP_TITLE");
		}

		protected override void OnBeginPrint(PrintEventArgs e) 
		{
			//PrintTicks("OnBeginPrint <start>: ");
			base.OnBeginPrint(e);
			noteID = 0;
			pageID = 1;
			//PrintTicks("OnBeginPrint <end>: ");
		}

		protected override void OnEndPrint(PrintEventArgs e)
		{
			base.OnEndPrint(e);
			
		}

		protected override void OnPrintPage(PrintPageEventArgs e)
		{
			base.OnPrintPage (e);
			//PrintTicks("OnPrintPage <start>: " );
			Graphics gdiPage = e.Graphics;
			float leftMargin = e.MarginBounds.Left;
			float topMargin = e.MarginBounds.Top;
			//float oneNoteLineHeight;
			float defaultPrintFontHeight = defaultPrintFont.GetHeight(gdiPage);
			float pointFromTop = topMargin;
			//int lines = 0;
			float headerLineHeight = 2; 
			SizeF printSize;
			
			stringFormat.FormatFlags =  StringFormatFlags.NoClip | StringFormatFlags.LineLimit | StringFormatFlags.MeasureTrailingSpaces;
		
			Pen linePen = new Pen(Color.Black, headerLineHeight);

			float pageHeight = e.MarginBounds.Height + topMargin;
			if (e.PageSettings.Margins.Bottom <= (defaultPrintFontHeight + BOTTOM_PAGE))
			{
				pageHeight -= defaultPrintFontHeight + BOTTOM_PAGE;
			}
			
			SizeF availableArea = new SizeF(e.MarginBounds.Width, pageHeight);

			while ((!e.HasMorePages) && (noteID < notes.Length))
			{	//get one note[noteID] line height
				Font f;
				if (null != notes[noteID].GetOwnFont)
					f = notes[noteID].GetOwnFont;
				else
					f = defaultPrintFont;

				//oneNoteLineHeight = f.GetHeight(gdiPage);

				if ("" == textForNextPage)
				{
					// measure notes[noteID] header height (line + header)
					availableArea.Height = pageHeight - (pointFromTop + headerLineHeight);
					string headerTextOver = "";
					//if header has to be printed then calculate its height
					if (notes[noteID].HasHeader)
					{
						string headerPrintText = notes[noteID].HeaderText;
						printSize = GetPrintedTextArea( ref headerPrintText, availableArea, e.Graphics, 
							defaultPrintFont, ref headerTextOver);
					} 
					//header has to be on same page as body
					if ("" != headerTextOver) 
					{
						e.HasMorePages = true;
						break;
					}
					// 
					// Print notes[noteID] header: line and header text (reminder or category, archive time
					// Print horizontal line
					gdiPage.DrawLine(linePen, leftMargin, pointFromTop, e.MarginBounds.Right, pointFromTop);
					pointFromTop += headerLineHeight;
					//Print header if it is set.
					if (notes[noteID].HasHeader)
					{
						availableArea.Height = pageHeight - pointFromTop;
						string headerPrintText = notes[noteID].HeaderText;
						printSize = GetPrintedTextArea( ref headerPrintText, availableArea, e.Graphics, 
							defaultPrintFont, ref headerTextOver);
						//gdiPage.DrawRectangle( new Pen(Color.Red, 1), leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height);

						StringFormat headerFormat = new StringFormat(stringFormat);
						headerFormat.Alignment = notes[noteID].HeaderAlign;
						
						gdiPage.DrawString( headerPrintText, defaultPrintFont, Brushes.Black, 
							new RectangleF(leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height),
							headerFormat); 
						pointFromTop += printSize.Height + 3; 
					}

					//
					// Printing of notes[noteID] text. This can be separated on a next page or split into more pages.
					//
					
					//compare complete desknote text height with remaining page height.
					// if page height is shorter then notes[noteID] needs to be divided.
					string printNow = notes[noteID].BodyText;
					availableArea.Height = pageHeight - pointFromTop;
					printSize = GetPrintedTextArea( ref printNow, availableArea, e.Graphics, 
						f, ref textForNextPage);
					if ("" != textForNextPage) 
						e.HasMorePages = true;
					else
						noteID++;
				
					//gdiPage.DrawRectangle( new Pen(Color.Blue, 1), leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height);
					// print text.
					gdiPage.DrawString(printNow, f, Brushes.Black, 
						new RectangleF(leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height),
						stringFormat); //StringFormat.GenericTypographic);
					pointFromTop += printSize.Height + 3;
				}
				else 
				{
					//////////////////////////////
					//
					// DeskNote's text continuous from previous page and will be printed
					// on this or more pages. DeskNote header is not printed in this case, 
					// it was printed on a previous page
					//
					//////////////////////////////
					string printNow = textForNextPage;
					textForNextPage = null;
					availableArea.Height = pageHeight - pointFromTop;
					printSize = GetPrintedTextArea( ref printNow, availableArea, e.Graphics, 
						f, ref textForNextPage);
					if ("" != textForNextPage)
						e.HasMorePages = true;
					else 
						noteID++;
					//gdiPage.DrawRectangle( new Pen(Color.Blue, 1), leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height);
					// print text for this page.
					gdiPage.DrawString(printNow, f, Brushes.Black, 
						new RectangleF(leftMargin, pointFromTop, e.MarginBounds.Width, printSize.Height),
						stringFormat); //StringFormat.GenericTypographic);
					pointFromTop += printSize.Height + 3;
				}
			}
			
			// 
			// Print page number at the bottom of the page
			//
			
			string pageInfo = string.Format(MainResource.GetString("PAGE"), pageID++);
			StringFormat footerFormat = new StringFormat(stringFormat);
			footerFormat.Alignment = StringAlignment.Center;
			gdiPage.DrawString( pageInfo, defaultPrintFont, Brushes.Black, 
				new RectangleF(leftMargin, pageHeight  + 2, e.MarginBounds.Width, defaultPrintFontHeight + 1),
				footerFormat);
			//PrintTicks( "PrintPage <return>: ");

		}


		/// <summary>
		/// Calculates area where text fits while printed compared to available area.
		/// </summary>
		/// <param name="text">Text to be printed into available area. Modified by method if area is smaller than text requires.</param>
		/// <param name="avalaibleArea">Avalable area on the page where text has to be printed</param>
		/// <param name="g">Graphis GDI</param>
		/// <param name="f">Font by which it is printed</param>
		/// <param name="textOver">Text which does not fit into available area and will be printed on a next page.</param>
		/// <returns>Size of the area where text fits in even it is shorter.</returns>
		internal SizeF GetPrintedTextArea(ref string text, SizeF availableArea, Graphics g, Font f, ref string textOver)
		{
			SizeF stringSize;
			int charsFitted, linesFilled;
			textOver = "";

			stringSize = g.MeasureString(text, f, availableArea, stringFormat, // StringFormat.GenericTypographic,
				out charsFitted, out linesFilled);
            // If charsFitted are zero then all text has to go over
			// If text does not fit there then cut it in first space or \n, \t before max chars that fits in.
			 if ((charsFitted < text.Length) && (0 < charsFitted))
			{
				textOver = text;
				string nL = " \n\t";
				char[] anyOf = nL.ToCharArray();
                // TODO - If three pages are printed then 2nd one is without page number and 3rd one has number 2.
				int spacePos = text.LastIndexOfAny( anyOf, charsFitted, charsFitted-1 );
				if (spacePos > 0)
				{
					text = text.Substring(0, spacePos);
					textOver = textOver.Substring(spacePos+1);
				} 
				else
				{
					text = text.Substring(0, charsFitted);
					textOver = textOver.Substring(charsFitted+1);
				}
            }
                 // nothing fits into remaining area on a page.
            else if (0 == charsFitted)
            {
                textOver = text;
                text = "";
            }
			// Sometimes last line is not printed. 
			// Adding 5% of font size helps in many cases
			stringSize.Height += f.GetHeight(g) / 20;
			
			return stringSize;
		}

		
	}
}
